/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file corners.cpp
@ingroup ExamplePrograms
@brief Obtains <a href="http://en.wikipedia.org/wiki/Corner_detection#The_Harris_.26_Stephens_.2F_Plessey_corner_detection_algorithm">Harris corners</a> from the frames of a video input source.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./corners --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>

#include <qvdta/qvpolyline.h>
#include <qvdta/qvdta.h>

/// \cond
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty("Window size", inputFlag, 10, "window size ", 1, 100);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Corner response image", outputFlag);
			addProperty< QVImage<uChar,3> >("Corners", outputFlag);
			}

		void iterate()
			{
			int sizeMax = getPropertyValue<int>("Window size");
		
			QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			uInt rows = image.getRows(), cols = image.getCols();
			timeFlag("grab Frame");
		
			////////////////////////////////////
			// Harris corner response image
			QVImage<sFloat>	eigval(cols, rows);
			HarrisCornerResponseImage(image, eigval);
			timeFlag("Corner response image");

			QVImage<uChar,1> eigvalUChar = eigval, equalizedEigval(cols, rows);
			equalizeHistogram(eigvalUChar, equalizedEigval);
		
			setPropertyValue< QVImage<uChar,1> >("Corner response image", equalizedEigval);
			timeFlag("Show corner response");
		
			////////////////////////////////////
			// Local maximal filter
			QVImage<uChar> hotpoints(cols, rows);
			FilterLocalMax(eigval, hotpoints, sizeMax, sizeMax);
			//setPropertyValue< QVImage<uChar,3> >("Corners", hotpoints);
			
			QList<QPoint> hotPoints;
			QVIMAGE_INIT_READ(uChar,hotpoints);
			for(uInt row = 0; row < hotpoints.getRows(); row++)
				for(uInt col = 0; col < hotpoints.getCols(); col++)
					if (QVIMAGE_PIXEL(hotpoints, col, row,0))
						hotPoints.append(QPoint(col, row));
			timeFlag("Local maximal filter");

			////////////////////////////////////
			QVImage<uChar,3> destino = image;
			drawPoints(hotPoints, destino);
			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("Draw corners");
		
			////////////////////////////////////
			/*QVImage<uChar> buffer;
			QList<QPoint> pointList;
			FindPeaks3x3GetBufferSize(eigval, buffer);		
			FindPeaks3x3(eigval, buffer, pointList, 0.0, ippiNormInf, 1, hotPoints.size());
			timeFlag("Find Peaks");

			////////////////////////////////////
			QVImage<uChar,3> destino = image;
			drawPoints(pointList, destino);
			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("Draw corners");
			*/
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Applies corner detection over an input video."

		);

	QVMPlayerCamera camera("Video");
	MyWorker worker("Corners Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Corners");
	imageCanvas.linkProperty(worker, "Corners");

	return app.exec();
	}

/// \endcond
