/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file corners.cpp
@ingroup ExamplePrograms
@brief Obtains <a href="http://en.wikipedia.org/wiki/Corner_detection#The_Harris_.26_Stephens_.2F_Plessey_corner_detection_algorithm">Harris corners</a> from the frames of a video input source.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./corners --URL=<path or url to video>
\endcode

En mathematica:
H = {{h1, h2, h3}, {h4, h5, h6}, {h7, h8, h9}};
Cross[{p, q, 1}, H .{x, y, 1}]

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <math.h>

#include <QDebug>
#include <QVector>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>

#include <qvdta/qvpolyline.h>
#include <qvdta/qvdta.h>

#include <TooN/TooN.h>
#include <TooN/numerics.h>	// Basic TooN library
#include <TooN/numhelpers.h>	// If you want a few extras
#include <TooN/SVD.h>		// If you want singular value decompositions
#include <TooN/LU.h>		// If you want lu triangular decompositions
#include <TooN/SymEigen.h>	// If you want Eigen decomposition of symmetric matrices

#define PI	3.14159

Q_DECLARE_METATYPE(Matrix<>);

/// \cond
void GetHotPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, uInt maxWindow)
	{
	const uInt rows = cornerResponseImage.getRows(), cols = cornerResponseImage.getCols();

	QVImage<uChar> binaryCornerImage(cols, rows);
	FilterLocalMax(cornerResponseImage, binaryCornerImage, maxWindow, maxWindow);

	QVIMAGE_INIT_READ(uChar,binaryCornerImage);
	for (uInt row = 0; row < binaryCornerImage.getRows(); row++)
		for (uInt col = 0; col < binaryCornerImage.getCols(); col++)
			if (QVIMAGE_PIXEL(binaryCornerImage, col, row,0))
				hotPoints.append(QPoint(col, row));
	}

void GetMaximalPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, QList<QPoint> &maximalPoints, uInt maxPoints)
	{
	while( hotPoints.size() > 0 && maximalPoints.size() < maxPoints )
		{
		uInt maxIndex = 0;
		for (int n=0; n < hotPoints.size(); n++)
			if ( cornerResponseImage(hotPoints.at(n)) >  cornerResponseImage(hotPoints.at(maxIndex)) )
				maxIndex = n;

		maximalPoints.append(hotPoints.at(maxIndex));
		hotPoints.removeAt(maxIndex);
		}
	}

uInt	getClosestPointIndex(const QPoint point, const QList<QPoint> &pointList)
	{
	uInt index = 0;
	for (uInt n = 1; n < pointList.size(); n++)
		if ((point - pointList.at(n)).manhattanLength() < (point - pointList.at(index)).manhattanLength())
			index = n;

	return index;
	}

QPoint	getMeanPoint(const QList<QPoint> &pointList)
	{
	QPoint center(0,0);

	for (uInt n = 0; n < pointList.size(); n++)
		{
		center.rx() += pointList.at(n).x();
		center.ry() += pointList.at(n).y();
		}

	center.rx() = center.rx() / pointList.size();
	center.ry() = center.ry() / pointList.size();

	return center;
	}

double	angle(const QPoint &p)
	{
	double x = p.x(), y = p.y();
	if (x>0)
		if (y>=0)
			return atan(y/x);
		else
			return atan(y/x) + 2*PI;
	else if (x == 0)
		if (y>0)
			return PI/2;
		else
			return 3*PI/2;
	else // x < 0
		return atan(y/x)+PI;
	}

double	clockWiseAngle(const QPoint &p1, const QPoint &p2)
	{
	double clockAngle = angle(p2) - angle(p1);
	return (clockAngle < 0)? clockAngle + 2*PI:clockAngle;
	}

// This function creates a list of the template points, first with the center point,
// then the rest of the points following a clockwise order around the center point.
QList<QPoint> GetTemplatePoints(uInt zoom, QPoint center)
	{
	QPoint rPoints[5] =
		//	Center		Norwest		Noreast		Southeast	Southwest
		{	QPoint(0,0),	QPoint(-1,+1),	QPoint(+1,+1),	QPoint(+1,-1),	QPoint(-1,-1)	};

	QList<QPoint> templatePoints;

	for (uInt i= 0; i < 5; i++)
		templatePoints.append(zoom*rPoints[i]+center);

	return templatePoints;
	}

// This function creates a list of the template points, first with the center point,
// then the rest of the points following a clockwise order around the center point.
Matrix<> GetTemplateMatrixPoints()
	{
	//					Center		Norwest		Noreast		Southeast	Southwest
	return Matrix<5,3> ((double[5][3]){	0,0,1,		-1,+1,+1,	+1,+1,+1,	+1,-1,+1,	-1,-1,+1});
	}

// This is suposed to sort the points in a list, by... who knows?
// Anyway, it doesn't works
bool SortTemplatePoints(QList<QPoint> &points)
	{
	if (points.size() != 5)
		return false;

	QList<QPoint> result;

	// This array will keep the index of the points.
	uInt index[5];

	// Calculate the index of the central point
	uInt indexp = getClosestPointIndex(getMeanPoint(points), points);

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the index of the top left point
	double minDistance = 1000000;
	for (uInt n = 0; n < points.size(); n++)
		if ( points.at(n).manhattanLength() < minDistance )
			{
			minDistance = points.at(n).manhattanLength();
			indexp = n;
			}

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the other points
	while(points.size() > 0)
		{
		indexp = 0;
		double minAngle = clockWiseAngle( result.back() - result.front(), points.at(indexp) - result.front());
	
		for (uInt n = 1; n < points.size(); n++)
			{
			double actualAngle = clockWiseAngle( result.back() - result.front(), points.at(n) - result.front());
			if ( actualAngle < minAngle )
				{
				minAngle = actualAngle;
				indexp = n;
				}
			}

		result.append(points.at(indexp));
		points.removeAt(indexp);
		}

	points = result;

	return true;
	}


// This function returns the homography that maps the points from a source position to a
// destination position
Matrix<> CalibrateHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints)
	{
	Q_ASSERT(sourcePointsMatrix.num_cols() == 2);
	Q_ASSERT(sourcePointsMatrix.num_cols() == destinationPoints.num_cols());
	Q_ASSERT(sourcePointsMatrix.num_rows() == destinationPoints.num_rows());

	const uInt rows = sourcePointsMatrix.num_rows();

	// First, state the coefs matrix for the homogeneous linear system of equations, of the form:
	//	Ax = 0
	Matrix<> coefsMatrix(3*rows,9);

	//std::cout << "Points" << std::endl;
	for (uInt n = 0; n < rows; n++)
		{
		double	x = sourcePointsMatrix[n][0], y = sourcePointsMatrix[n][1],
			p = destinationPoints[n][0], q = destinationPoints[n][1];

		double	equation1[9] = {	0,	0,	0,	-x,	-y,	-1,	q*x,	q*y,	q},
			equation2[9] = {	x,	y,	1,	0,	0,	0,	-p*x,	-p*y,	-p},
			equation3[9] = {	-q*x,	-q*y,	-q,	p*x,	p*y,	p,	0,	0,	0};

		coefsMatrix[3*n] = Vector<9>(equation1);
		coefsMatrix[3*n+1] = Vector<9>(equation2);
		coefsMatrix[3*n+2] = Vector<9>(equation3);
		}

	// create the SVD of M
	SVD<> svdCoefsMatrix(coefsMatrix);

	Vector<9> x =  svdCoefsMatrix.get_VT()[8];
	Matrix<> homography(3,3);

	homography[0][0] = x[0];	homography[0][1] = x[1];	homography[0][2] = x[2];
	homography[1][0] = x[3];	homography[1][1] = x[4];	homography[1][2] = x[5];
	homography[2][0] = x[6];	homography[2][1] = x[7];	homography[2][2] = x[8];

	return homography;
	}

// This function maps a point given a planar homography
QPoint	applyHomography(const QPoint p, const Matrix<> homography)
	{
	const double values[3] = { p.x(), p.y(), 1 };
	Vector<3> p1 = Vector<3>(values), p2 = homography * p1;
	return QPoint(round(p2[0]/p2[2]), round(p2[1]/p2[2]));
	}

void normalizeHomogeneousCoordinates(Matrix<> &points)
	{
	const uInt cols = points.num_cols(), rows = points.num_rows();

	for (uInt i = 0; i < rows; i++)
		for (uInt j = 0; j < cols; j++)
			points[i][j] /= points[i][cols-1];
	}

// This function retunrs an integer value that indicates how well the homography maps the source points
// in a list, to the points in the destination list.
double	testErrorHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints, const Matrix<> homography)
	{
	const uInt cols = sourcePointsMatrix.num_cols(), rows = sourcePointsMatrix.num_rows();

	Matrix <> projectedPoints(sourcePointsMatrix.num_rows(),3);
	Matrix <> residuals (sourcePointsMatrix.num_rows(),3);

	projectedPoints = sourcePointsMatrix * homography.T();
	normalizeHomogeneousCoordinates(projectedPoints);
	residuals = projectedPoints - destinationPoints;

	double accum = 0;
	for (uInt i = 0; i < rows; i++)
		{
		double square = 0;
		for (uInt j = 0; j < cols; j++)
			square += residuals[i][j]*residuals[i][j];
		accum += sqrt(square);
		}

	return accum;
	}

// This function sustitudes warpPerspective from the qvipp, and applies the planar transformation H
// to a source image, storing the transformed image in the destination image
void	myWarpPerspective(const QVImage<uChar> &src, QVImage<uChar> &dest, const Matrix <> H, int zoom)
	{
	const uInt cols = src.getCols(), rows = src.getRows();
	const QPoint center = QPoint(cols/2, rows/2);
	const Matrix<> Hinv = SVD<>(H).get_pinv();
	const double meanColsRows = ((double)(cols + rows)) / 2;

	for (uInt col = 0; col < cols; col++)
		for (uInt row = 0; row < rows; row++)
			{
			Vector<3> v;
			v[0] = col - cols/2;
			v[1] = row - rows/2;
			v[2] = meanColsRows;
			Vector<3>vP;
			vP = H*v;
			QPoint p2 =  QPoint(round(zoom*vP[0]/vP[2]),round(zoom*vP[1]/vP[2]))+center, p1 = QPoint(col, row);
			if (dest.getROI().contains(p2))
				dest(p2) = src(p1);
			}
	}

// Given the four corners of a square, transformed by an homography, this function obtains two points
// of the line of the horizont.
bool getHorizont(const double x1, const double y1, const double x2, const double y2, const double x3, const double y3, const double x4, const double y4,
		double &x5, double &y5, double &x6, double &y6)
	{
	const double	x5num =		x1 *(x4 *(-y2 + y3) + x3 *(y2 - y4)) + x2 *(x4 *(y1 - y3) + x3 *(-y1 + y4)),
			x5denom =	x4 *(y1 - y2) + x3 *(-y1 + y2) + (x1 - x2)* (y3 - y4),
			y5num =		x4 *(y1 - y2)* y3 + x1* y2* y3 - x3* y1* y4 - x1* y2* y4 + x3 *y2* y4 + x2 *y1 *(-y3 + y4),
			y5denom =	x4 *(y1 - y2) + x3 *(-y1 + y2) + (x1 - x2)* (y3 - y4),

			x6num =		x3 *(x4 *(y1 - y2) + x1 *(y2 - y4)) + x2* (x4 *(-y1 + y3) + x1 *(-y3 + y4)),
			x6denom =	(x1 - x4) *(y2 - y3) + x3 *(y1 - y4) + x2 *(-y1 + y4),
			y6num =		-x2* y1* y3 + x4* y1* (-y2 + y3) + x3* y2* (y1 - y4) + x1* y2* y4 - x1* y3* y4 + x2* y3* y4,
			y6denom =	(x1 - x4)* (y2 - y3) + x3* (y1 - y4) + x2* (-y1 + y4);

	if (x5denom == 0 || y5denom == 0 || x6denom == 0 || y6denom == 0)
		return false;

	x5 = x5num / x5denom;
	y5 = y5num / y5denom;
	x6 = x6num / x6denom;
	y6 = y6num / y6denom;

	return true;
	}

// This function gets the closest point in a line, to a given point, not contained in it
bool getPerpendicularPoint(const double xc, const double yc, const double x5, const double y5, const double x6, const double y6, double &x7, double &y7)
	{
	const double	x7num =		xc*x5*x5 + x6*(xc*x6-(yc-y5)*(y5-y6)) + x5*(-2*xc*x6+(yc-y6)*(y5-y6)),
			x7denom =	x5*x5-2*x5*x6+x6*x6+(y5-y6)*(y5-y6),
			y7num =		x6*x6*y5+yc*(y5-y6)*(y5-y6)+x5*x5*y6+xc*x6*(-y5+y6)+x5*((xc-x6)*y5-(xc+x6)*y6),
			y7denom =	x5*x5 - 2*x5*x6 + x6*x6 + (y5-y6)*(y5-y6);


	if (x7denom == 0 || y7denom == 0)
		return false;

	x7 = x7num / x7denom;
	y7 = y7num / y7denom;

	return true;

	}

// This function draws a line in an image, given two points of it.
void	myDrawLine(QVImage<uChar,3> &image, const int x1, const int y1, const int x2, const int y2)
	{
	const uInt rows = image.getRows(), cols = image.getCols();
	const double m = (double)(y2-y1)/(double)(x2-x1);

	for(int x = 0; x< cols; x++)
		{
		int y = m*(x - x1) + y1;
		if ( y >= 0 && y < rows)
			{
			image(x,y,0) = 255;
			image(x,y,1) = 0;
			image(x,y,2) = 0;
			}
		}
	}

// This function transforms a list of n points, to a nx3 matrix, containing the rows the
// homogeneous coordinates for each point in the list.
void pointListToMatrix(const QList<QPoint> &points, Matrix<> &matrix)
	{
	Q_ASSERT(points.size() == matrix.get_rows());
	Q_ASSERT(3 == matrix.get_cols());

	for (uInt n = 0; n < points.size(); n++)
		{
		double	v[3] = { points.at(n).x(), points.at(n).y(), 1 };
		matrix[n] = Vector<3>(v);
		}
	}

#define NORM(x,y)	(sqrt((x)*(x) + (y)*(y)))

void getSubpixelPrecissionPoints(const QVImage<sFloat> &weightImage, QList<QPoint> &pointList, Matrix<> &pointsMatrix, uInt margin)
	{
	const uInt	rows = weightImage.getRows(), cols = weightImage.getCols();

	QVIMAGE_INIT_READ(sFloat, weightImage);
	for (uInt n = 0; n < pointList.size(); n++)
		{
		const QPoint p = pointList.at(n);
		const int x = p.x(), y = p.y();
		if (x > margin && x < cols-margin && y > margin && y < rows-margin)
			{
			double v[3] = {0, 0, 1}, sum = 0;
			for (uint i = x-margin; i<=x+margin; i++)
				for (uint j = y-margin; j<=y+margin; j++)
					{
					const double factor = QVIMAGE_PIXEL(weightImage, i, j, 0);
					v[0] += i*factor;
					v[1] += j*factor;
					sum += factor;
					}

			v[0] = v[0] / sum;
			v[1] = v[1] / sum;

			pointsMatrix[n] = Vector<3>(v);
			}
		}
	}

// Gets the minimum point of the parabol that contains the three points (x1,y1), (x2,y2), (x3,y3). In Mathematica:
// Solve[(2a x + b /. Solve[{	a x1^2 + b  x1 + c == y1,
//				a x2^2 + b  x2 + c == y2,
//				a x3^2 + b  x3 + c == y3	},
//	{a, b, c}]) == 0, x]
double minimumAtCuadraticInterpolation(const double x1, const double y1, const double x2, const double y2, const double x3, const double y3)
	{
	const double	numerator = x2*x2*y1 - x3*x3*y1 - x1*x1*y2 + x3*x3*y2 + x1*x1*y3 - x2*x2*y3,
			denominator =  x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3;

	return numerator/(2*denominator);
	}

#define	POW2(X)	((X)*(X))
#define	ABS(X)	((X>0)?(X):(-X))

// Error function, given an 'f', and a 'H' matrix. In Mathematica:
//	K = {{f, 0, 0}, {0, f, 0}, {0, 0, 1}};
//	Hvar = {{h1, h2, h3}, {h4, h5, h6}, {h7, h8, h9}};
//	ErrorMat[H_, K_] := Transpose[H].Inverse[K].Inverse[K].H;
//	Error [ErrorM_] := ((ErrorM[[1, 1]] - ErrorM[[2, 2]])^2 + ErrorM[[1, 2]]^ 2) / ErrorM[[1, 1]]^ 2;
//	ErrorFunction[H_, K_] := Error[ErrorMat[H, K]];
//	ErrorFunction[Hvar, K] // Simplify
#define	ERROR(f) 								\
	(	(								\
		POW2( h1*h2 + h4*h5 + (f)*(f)*h7*h8 ) +				\
		POW2( h1*h1 - h2*h2 + h4*h4 - h5*h5 + (f)*(f)*(h7*h7-h8*h8) )	\
		) / POW2(h1*h1 + h4*h4 + (f)*(f)*h7*h7)				\
	)
double errorHomographyFocalDistance(const Matrix <3,3> &H, const double f)
	{
	const double	h1 = H[0][0],	h2 = H[0][1],	h3 = H[0][2],
			h4 = H[1][0],	h5 = H[1][1],	h6 = H[1][2],
			h7 = H[2][0],	h8 = H[2][1],	h9 = H[2][2];
	return ERROR(f);
	}

double errorHomographiesFocalDistance(const QVector< Matrix <3,3> > &Hlist, const double f)
	{
	double error = 0;
	for (uInt n = 0; n< Hlist.size(); n++)
		{
		Matrix<3,3> H = Hlist[n];
		const double	h1 = H[0][0],	h2 = H[0][1],	h3 = H[0][2],
				h4 = H[1][0],	h5 = H[1][1],	h6 = H[1][2],
				h7 = H[2][0],	h8 = H[2][1],	h9 = H[2][2];
		error +=	( ( POW2( h1*h2 + h4*h5 + (f)*(f)*h7*h8 ) + POW2( h1*h1 - h2*h2 + h4*h4 - h5*h5 + (f)*(f)*(h7*h7-h8*h8) ) ) / POW2(h1*h1 + h4*h4 + (f)*(f)*h7*h7) );
		}

	return error;
	}

// This function finds the fov that minimices errorHomographyFocalDistance for a given homography H,
// in a neighbourhood of a fov value passed in the variable 'startingFov'.
double calibrateFovDistance(const Matrix<> &Horig, const double startingFov = 3)
	{
	Matrix <3,3> H;

	// We will work with the inverse of Horig
	H = SVD<>(Horig).get_pinv();
	double f = startingFov;
	
	for (uInt i = 0; i < 10; i++)
		{
		f = minimumAtCuadraticInterpolation(
					f, errorHomographyFocalDistance(H,f),
					f-0.1, errorHomographyFocalDistance(H,f-0.1),
					f+0.1, errorHomographyFocalDistance(H,f+0.1)
					);
		}
	//std::cout << i << " f = " << f << ", ERROR = " << errorHomographyFocalDistance(H,f) << std::endl;
	return f;
	}

bool minimizeFovErrorFunction(const QVector< Matrix<3,3> > &H, double &f, const uInt maxSteps = 100)
	{
	double startError = errorHomographiesFocalDistance(H,f);
	double lastF = f, lastError = startError;

	for (uInt i = 0; i < maxSteps; i++)
		{
		f = minimumAtCuadraticInterpolation(
					f, errorHomographiesFocalDistance(H,f),
					f-0.1, errorHomographiesFocalDistance(H,f - 0.1),
					f+0.1, errorHomographiesFocalDistance(H,f + 0.1)
					);

		const double error = errorHomographiesFocalDistance(H,f);

		std::cout << i << " f = " << f << ", ERROR = " << error << std::endl;

		if (error > startError)
			return false;

		if (error > lastError)
			{
			f = lastF;
			return true;
			}

		lastF = f;
		lastError = error;
		}

	return false;
	}

double calibrateFovDistance(const QVector< Matrix<3,3> > &Horig, const uInt maxSteps = 100)
	{
	QVector< Matrix<3,3> > H;
	H.resize(Horig.size());

	
	// We will work with the inverse of Horig's
	for(uInt n=0; n< Horig.size(); n++)
		H[n] = SVD<>(Horig[n]).get_pinv();

	for (double f0 = 0.5; f0 < 10; f0 += 0.5)
		{
		double f = f0;
		if (minimizeFovErrorFunction(H, f, maxSteps))
			return f;
		}

	return 0.0;
	}

#define	MAX_BEST_MATRICES	12

class MyWorker: public QVWorker
	{
	//Matrix<3,3> matrixList[MAX_BEST_MATRICES];
	double	matrixError[MAX_BEST_MATRICES];
	QVector< Matrix<3,3> > matrixList;

	public:
		MyWorker(QString name): QVWorker(name)
			{
			matrixList.resize(MAX_BEST_MATRICES);
			for(uInt n=0; n<MAX_BEST_MATRICES; n++)
				matrixError[n] = 100;

			addProperty<double>("Max error", inputFlag, 0.02, "window size", 0, 0.1);
			addProperty<double>("Search starting FOV", inputFlag, 3.5, "window size", 0, 16);

			addProperty<int>("Zoom", inputFlag, 50, "window size", 1, 256);
			addProperty<int>("Window size", inputFlag, 10, "window size", 1, 100);
			addProperty<int>("Margin", inputFlag, 1, "window size", 1, 64);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("Corners", outputFlag);
			addProperty< QVImage<uChar,1> >("Wrapped", outputFlag);
			addProperty< QVImage<uChar,1> >("Wrapped2", outputFlag);
			addProperty< QVImage<sFloat,1> >("Response image", outputFlag);

			addTrigger("Grab homography");
			addProperty< Matrix<> >("Actual homography", outputFlag);
			}

		void processTrigger(QString triggerName)
			{
			/*std::cout <<	"H = {"
					<< "{" << H[0][0] << ", " << H[0][1] << ", " << H[0][2] << "},"
					<< "{" << H[1][0] << ", " << H[1][1] << ", " << H[1][2] << "},"
					<< "{" << H[2][0] << ", " << H[2][1] << ", " << H[2][2] << "}};" << std::endl;

			std::cout << "--------- Best homographies ----------" << std::endl;

			for (uInt n=0; n<MAX_BEST_MATRICES; n++)
				{
				Matrix<3,3> H;
				H = matrixList[n];
				std::cout <<	"H" << n <<" = {"
					<< "{" << H[0][0] << ", " << H[0][1] << ", " << H[0][2] << "},"
					<< "{" << H[1][0] << ", " << H[1][1] << ", " << H[1][2] << "},"
					<< "{" << H[2][0] << ", " << H[2][1] << ", " << H[2][2] << "}}; (* " << matrixError[n] << "*)" << std::endl;
				}*/

			//std::cout << "startingFov = " << f << std::endl;
			double fov = calibrateFovDistance(matrixList);
			std::cout << "FOV[" << MAX_BEST_MATRICES << "] = " << fov << std::endl;
			std::cout << "FOV = " << calibrateFovDistance(getPropertyValue< Matrix<> >("Actual homography"),fov+1) << std::endl;
			};

		void iterate()
			{
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			const uInt	rows = image.getRows(), cols = image.getCols(),
					sizeMax = getPropertyValue<int>("Window size"),
					zoom = getPropertyValue<int>("Zoom"),
					margin = getPropertyValue<int>("Margin");
			const double	maxError = getPropertyValue<double>("Max error");

			QVImage<uChar,3> destino = image;

			timeFlag("grab Frame");

			////////////////////////////////////
			// Harris corner response image
			QVImage<sFloat>	temp(cols, rows), cornerResponseImage(cols, rows);
			//HarrisCornerResponseImage(image, cornerResponseImage);
			SobelCornerResponseImage(image, cornerResponseImage);

			timeFlag("Corner response image");

			//setPropertyValue< QVImage<sFloat> >("Response image", cornerResponseImage);

			////////////////////////////////////
			// Hot points
			QList<QPoint> hotPoints;
			GetHotPoints(cornerResponseImage, hotPoints, sizeMax);

			timeFlag("Get hotpoints");

			///////////////////////////////////////////
			// Calibración
			QList<QPoint> maximalPoints, templatePoints = GetTemplatePoints(1, QPoint(0, 0));
			GetMaximalPoints(cornerResponseImage, hotPoints, maximalPoints, 5);
			SortTemplatePoints(maximalPoints);

			timeFlag("Get max hotpoints");

			if (maximalPoints.size() == 5)
				{
				Matrix <> sourcePointsMatrix(5,3), sourcePointsMatrix2(5,3), destinationPointsMatrix(5,3);
				destinationPointsMatrix =  GetTemplateMatrixPoints();
				getSubpixelPrecissionPoints(cornerResponseImage, maximalPoints, sourcePointsMatrix,margin);
				pointListToMatrix(maximalPoints, sourcePointsMatrix2);

				//pointListToMatrix(templatePoints, destinationPointsMatrix);

				const double meanColsRows = cols/2;
				for (uInt i = 0; i<5; i++)
					{
					sourcePointsMatrix[i][0] = (sourcePointsMatrix[i][0] - cols/2)/ meanColsRows;
					sourcePointsMatrix[i][1] = -(sourcePointsMatrix[i][1] - rows/2)/ meanColsRows;
					sourcePointsMatrix2[i][0] = (sourcePointsMatrix2[i][0] - cols/2)/ meanColsRows;
					sourcePointsMatrix2[i][1] = -(sourcePointsMatrix2[i][1] - rows/2)/ meanColsRows;
					}

				/////////////////////////

				Matrix<> H = CalibrateHomography(sourcePointsMatrix, destinationPointsMatrix);
				Matrix<> H2 = CalibrateHomography(sourcePointsMatrix2, destinationPointsMatrix);

				/*std::cout << "--------------------------" << std::endl;
				std::cout << "ERROR 1 " << testErrorHomography(sourcePointsMatrix, destinationPointsMatrix, H) << std::endl;
				std::cout << "ERROR 2 " << testErrorHomography(sourcePointsMatrix2, destinationPointsMatrix, H2) << std::endl;*/

				const double actualError = testErrorHomography(sourcePointsMatrix, destinationPointsMatrix, H);
				if (actualError < maxError)
					{
					setPropertyValue< Matrix<> >("Actual homography", H);

					uInt index = 0;
					for (uInt i=1; i<MAX_BEST_MATRICES; i++)
						if (matrixError[i] > matrixError[index])
							index = i;

					if (matrixError[index] > actualError)
						{
						matrixList[index] = H;
						matrixError[index] = actualError;
						}


					QVImage<uChar> wrapped(cols, rows);
					Set(wrapped,0);
	
					myWarpPerspective(image, wrapped, H, zoom);

					//////////////////////////////
					// Warp perspective using IPP 
					/*double coeffs[3][3]; 
					for (uInt i = 0; i<3; i++) 
						for (uInt j=0; j<3; j++) 
							coeffs[i][j] = H[i][j]; 
					WarpPerspective(image, wrapped, coeffs);*/
					//////////////////////////////

					QVImage<uChar> wrapped2(cols, rows);
					Set(wrapped2,0);
	
					myWarpPerspective(image, wrapped2, H2,zoom);

					QVImage<uChar> diff(cols, rows);
					AbsDiff(wrapped, wrapped2, diff);
					setPropertyValue< QVImage<uChar,1> >("Wrapped", wrapped);
					setPropertyValue< QVImage<uChar,1> >("Wrapped2", wrapped2);

					drawPoints(maximalPoints, destino);
					//std::cout << "FOV = " << calibrateFovDistance(H) << std::endl;
					}
				}

			timeFlag("Calibrate");

			// Fin rectificación ////////////////////////

			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("Draw corners");
		
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Applies corner detection over an input video."

		);

	QVMPlayerCamera camera("Video");
	MyWorker worker("Corners Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Corners");
	imageCanvas.linkProperty(worker, "Corners");

	QVImageCanvas imageCanvas2("Wrapped");
	imageCanvas2.linkProperty(worker, "Wrapped");

	QVImageCanvas imageCanvas4("Wrapped 2");
	imageCanvas4.linkProperty(worker, "Wrapped2");

	return app.exec();
	}

/// \endcond


///////////////////////////////////////
/*QVImage<uChar> buffer;
QList<QPoint> pointList;
FindPeaks3x3GetBufferSize(cornerResponseImage, buffer);		
FindPeaks3x3(cornerResponseImage, buffer, pointList, 0.0, ippiNormInf, 1, hotPoints.size());
timeFlag("Find Peaks");*/

// Solucion en el Mathematica:
//	f = a0 + a1 x + a2 y + a3 x y + a4 x^2 + a5 y^2;
//	Solve[{D[f, x] == 0, D[f, y] == 0} // Simplify, {x, y}] // Simplify
/*bool minimumAtCuadraticInterpolation(const Matrix <> &X, const Vector<> &y, Vector<> &xDest)
	{
	Matrix<> A(X.num_rows(),6);

	for (uInt n = 0; n < X.num_rows(); n++)
		{
		const double	x0 = X[n][0], x1 = X[n][1];
		double		row[6] = {	1,	x0,	x1,	x0*x1,	x0*x0, x1*x1	};
		A[n] = Vector<6>(row);
		}

	Vector<> a = SVD<>(A).get_pinv()*y;

	const double denom = a[3]*a[3] - 4*a[4]*a[5];
	if (denom == 0)
		return false;

	xDest[0] = (-a[2]*a[3]+2*a[1]*a[5]) / denom;
	xDest[1] = (-a[1]*a[3]+2*a[2]*a[4]) / denom;

	return true;
	}

// Solucion en el Mathematica:
//	f = a0 + a1 x + a2 y + a3 x^2 + a4 y^2;
//	Solve[{D[f, x] == 0, D[f, y] == 0} // Simplify, {x, y}] // Simplify
bool minimumAtCuadraticInterpolation2(const Matrix <> &X, const Vector<> &y, Vector<> &xDest)
	{
	Matrix<> A(X.num_rows(),5);

	for (uInt n = 0; n < X.num_rows(); n++)
		{
		const double	x0 = X[n][0], x1 = X[n][1];
		double		row[5] = {	1,	x0,	x1,	x0*x0, x1*x1	};
		A[n] = Vector<5>(row);
		}

	Vector<> a = SVD<>(A).get_pinv()*y;

	xDest[0] = -a[1]/(2*a[3]);
	xDest[1] = -a[2]/(2*a[4]);

	return true;
	}*/

// Given a pixel in an image, this function gets the coordinates of the points and the gray-level value
// in the image of the points in a square neighbourhood of the pixel, of size 2*margin+1
/*bool getMatrices(const QVImage<sFloat> &image, const QPoint p, Matrix<> &X, Vector<> &y, uInt margin)
	{
	const uInt	cols = image.getCols(), rows = image.getRows(),
			col = p.x(), row = p.y();

	if (col < margin || col > cols -margin || row < margin || row > rows -margin)
		return false;

	for (uInt i = col - 2, index = 0; i <= col +2; i++)
		for (uInt j = row - 2; j <= row +2; j++, index++)
			{
			double row[2] = {i, j};
			X[index] = Vector<2>(row);
			y[index] = image(i, j);
			}

	return true;
	}

void getSubpixelPrecissionPoints2(const QVImage<sFloat> &weightImage, QList<QPoint> &pointList, Matrix<> &pointsMatrix)
	{
	for (uInt n = 0; n < maximalPoints.size(); n++)
		{
		Matrix<> X(25,2);
		Vector <> y(25), x0(2);
		getMatrices(cornerResponseImage, maximalPoints.at(n), X, y,0);

		if (minimumAtCuadraticInterpolation2(X, y, x0))
			sourcePointsMatrix[n] = x0;
		else	{
			sourcePointsMatrix[n][0] = -100;
			sourcePointsMatrix[n][1] = -100;
			}
		//std::cout << "Matrix X" << X << std::endl;
		//std::cout << "Matrix y" << y << std::endl;
		
		std::cout << "vector " << maximalPoints.at(n).x() << ", " << maximalPoints.at(n).y() << "\t" << sourcePointsMatrix[n] << std::endl;
		}
*/

/*// This function retunrs an integer value that indicates how well the homography maps the source points
// in a list, to the points in the destination list.
uInt	testErrorHomography(const QList<QPoint> &sourcePointsMatrix, const QList<QPoint> &destinationPoints, const Matrix<> homography)
	{
	uInt accum = 0;
	for (uInt i = 0; i < sourcePointsMatrix.size(); i++)
		accum += (applyHomography(sourcePointsMatrix.at(i), homography) -destinationPoints.at(i)).manhattanLength();

	return accum;
	}*/
