/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file mser.cpp
@ingroup ExamplePrograms
@brief Obtains <a href="http://en.wikipedia.org/wiki/Maximally_stable_extremal_regions">maximally stable extremal regions</a> from frames of the input video source.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./mser --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>

#include <qvdta/qvmser.h>

/// \cond
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty<int>("Delta", inputFlag, 1, "MSER parameter, as seen in the paper.", 1, 128);
			addProperty<int>("minAreaMSER", inputFlag, 10, "MSER with area lesser than this value are discarted.", 1, 100*100);
			addProperty<int>("maxAreaMSER", inputFlag, 1000, "MSER with area greater than this value are discarted.", 1, 100*100);
			addProperty<double>("diffAreaThreshold", inputFlag, 0.01, "Proportion of the difference in areas to be considered different", 0.0, 1.0);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("MSER regions", outputFlag);
			}

		void iterate()
			{
			//////////////////
			// Read parameters
			const int delta = getPropertyValue<int>("Delta"),
				minArea = getPropertyValue<int>("minAreaMSER"),
				maxArea = getPropertyValue<int>("maxAreaMSER");
			const double diffAreaThreshold = getPropertyValue<double>("diffAreaThreshold");
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			
			timeFlag("Read parameters");

			///////////////////////
			// Inverted image
			const uInt rows = image.getRows(), cols = image.getCols();
			QVImage<uChar> notImage(cols, rows);
			for (uInt col = 0; col < cols; col++)
				for (uInt row = 0; row < rows; row++)
					notImage(col, row) = 255 - image(col, row);

			////////////////////
			// MSER
			QList<QVMSER> MSERListLow, MSERListHigh;

			getMSER(image, MSERListLow, delta, minArea, maxArea, diffAreaThreshold);
			timeFlag("MSER Low");

			getMSER(notImage, MSERListHigh, delta, minArea, maxArea, diffAreaThreshold);
			timeFlag("MSER High");

			////////////////////
			// Draw MSER's
			QList< QVPolyline > polylineMSERList;
			getMSERContours(image, MSERListLow, polylineMSERList);
			getMSERContours(notImage, MSERListHigh, polylineMSERList);

			QVImage<uChar,3> mserImage = image;
			draw(mserImage, polylineMSERList, (uChar []){ 255, 0, 0 });

			/////////////////
			// Publish resulting images
			setPropertyValue< QVImage<uChar,3> >("MSER regions",mserImage);
			timeFlag("Publish resulting images");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Gets component tree from images."
		);
	
	QVMPlayerCamera camera("Video");
	MyWorker worker("MSER Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("MSER Regions");
	imageCanvas.linkProperty(worker, "MSER regions");

	return app.exec();
	}

/// \endcond

	
